@@ -1,7 +1,7 @@
 /* Array bitsets.
 
-   Copyright (C) 2002, 2003, 2006, 2009, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 2002-2003, 2006, 2009-2012 Free Software Foundation,
+   Inc.
 
    Contributed by Michael Hayes (m.hayes@elec.canterbury.ac.nz).
 
